/*                
 *                        Licensing Notification
 *
 * Permission to use, reproduce, prepare derivative works, and to redistribute
 * to others this software and its documentation is hereby granted, provided
 * that this notice is retained thereon and on all copies or modifications. UChicago
 * Argonne, LLC, Tech-X Corporation, and all other contributors make no representations
 * as to the suitability and operability of this software for any purpose. It is
 * provided "as is" without express or implied warranty.
 *
 */

#ifndef CUDA_ERROR_UTILS_H
#define CUDA_ERROR_UTILS_H

// -----------------------------------------------------------------------------

#include <cuda_runtime_api.h>
#include <stdexcept>
#include <string>
#include <sstream>

// -----------------------------------------------------------------------------

class cuda_exception : public std::runtime_error
{
public:

	explicit cuda_exception(const std::string& mesg)
		: std::runtime_error(mesg)
	{
	}
};

// -----------------------------------------------------------------------------

class cuda_error_policy
{
public:

	static void throw_on_error(cudaError_t cuda_ret, const char* file, int line)
	{
		if(cuda_ret != cudaSuccess)
		{
			const char* cuda_str = cudaGetErrorString(cuda_ret);

			std::ostringstream ostr;
			ostr << "CUDA Error at " << file << ":" << line << ". " << cuda_str << "\n";

			throw cuda_exception(ostr.str());
		}
	}
};

// -----------------------------------------------------------------------------

#define check_if_cuda_error(cuda_ret) cuda_error_policy::throw_on_error(cuda_ret, __FILE__, __LINE__)
#define cuda_checked_call(expr)       cuda_error_policy::throw_on_error(expr, __FILE__, __LINE__)

// -----------------------------------------------------------------------------

#endif // CUDA_ERROR_UTILS_H
